// Copyright (C) 2006 Erik Dahlberg
// 
// This file is part of LSystem3D.
// 
// LSystem3D is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// LSystem3D is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with LSystem3D; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA




#ifndef GUI_H
#define GUI_H

#include <string>

#include "fx.h"

#include "lindenmayersystem.h"
#include "lsystemparameters.h"
#include "renderingsurface.h"

using namespace std;



/**
 * The GUI window
 */
class GUI : public FXMainWindow
{
    // macro to set up class declaration
    FXDECLARE(GUI)
    
public:
    
    // message ID's
    enum
    {
        ID_GENERATE = FXMainWindow::ID_LAST,
        ID_LOAD,
        ID_SAVE,
        ID_QUIT,
        ID_HELP_USAGE,
        ID_HELP_RULES,
        ID_LAST
    };
    
    
    /**
     * Constructor
     * @param application the application object
     * @param renderingSurface the rendering surface
     * @param lsystem the Lindenmayer system generator
     */
    GUI(FXApp *application, RenderingSurface *renderingSurface, LindenmayerSystem *lsystem);
    
    /**
     * Destructor
     */
    virtual ~GUI();
    
    
    /**
     * Create and initialize the window
     */
    virtual void create();
    
    
    /**
     * Called by the system when the "Generate" button is pressed
     * @param sender the sender object
     * @param selector message type and id
     * @param data event related data
     * @return 
     */
    long onGenerate(FXObject *sender, FXSelector selector, void *data);
    
    /**
     * Called by the system when the File->Load menu command is selected
     * @param sender the sender object
     * @param selector message type and id
     * @param data event related data
     * @return 
     */
    long onLoad(FXObject *sender, FXSelector selector, void *data);
    
    /**
     * Called by the system when the File->Save menu command is selected
     * @param sender the sender object
     * @param selector message type and id
     * @param data event related data
     * @return 
     */
    long onSave(FXObject *sender, FXSelector selector, void *data);
    
    /**
     * Called by the system when the close button or the File->Quit menu command is selected
     * @param sender the sender object
     * @param selector message type and id
     * @param data event related data
     * @return 
     */
    long onQuit(FXObject *sender, FXSelector selector, void *data);
    
    /**
     * Called by the system when the Help->Usage menu command is selected
     * @param sender the sender object
     * @param selector message type and id
     * @param data event related data
     * @return 
     */
    long onHelpUsage(FXObject *sender, FXSelector selector, void *data);
    
    /**
     * Called by the system when the Help->Rules menu command is selected
     * @param sender the sender object
     * @param selector message type and id
     * @param data event related data
     * @return 
     */
    long onHelpRules(FXObject *sender, FXSelector selector, void *data);
    
protected:
    
    /**
     * Constructor
     */
    GUI() {}
    
    
    /**
     * Load L-system from file and sync with GUI
     * @param filename path of the L-system file
     */
    void loadLSystem(string filename);
    
    
    // GUI widgets
    
    FXMenuPane *_fileMenuPane;  // File menu
    FXMenuPane *_helpMenuPane;  // Help menu
    
    FXTextField *_axiomTextField;       // Axiom
    FXText *_rulesText;                 // Rules
    FXRealSpinner *_angleRealSpinner;   // Angle
    FXSpinner *_depthSpinner;           // Depth
    
    FXRealSpinner *_diameterRealSpinner;        // Diameter
    FXRealSpinner *_diameterFactorRealSpinner;  // Diameter factor
    
    FXMessageBox *_helpUsageMessageBox; // Usage help dialog
    FXMessageBox *_helpRulesMessageBox; // Rules help dialog
    
    
    LindenmayerSystem *_lsystem;            // The Lindenmayer system generator
    LSystemParameters _lsystemParameters;   // Saver and loader for L-system parameters
    RenderingSurface *_renderingSurface;    // The rendering surface
};



#endif
