// Copyright (C) 2006 Erik Dahlberg
// 
// This file is part of LSystem3D.
// 
// LSystem3D is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// LSystem3D is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with LSystem3D; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA




#ifndef LINDENMAYERSYSTEM_H
#define LINDENMAYERSYSTEM_H

#include <string>

#include "model.h"
#include "turtle.h"
#include "rule.h"
#include "ruleset.h"

using namespace std;



/**
 * Lindenmayer System
 */
class LindenmayerSystem
{
public:
    
    /**
     * Constructor
     * @param model empty model
     */
    LindenmayerSystem(Model *model);

    /**
     * Destructor
     */
    ~LindenmayerSystem();
    
    /**
     * Clear all parameters
     */
    void clear();
    
     /**
     * Generate L-system data
     */
    void generate();
    
    /**
     * Set the initial rule (the axiom)
     * @param axiom the axiom
     */
    void setAxiom(Rule axiom);
    
    /**
     * Set one rule
     * @param rule the rule
     */
    void setRule(Rule rule);
    
    /**
     * Set the turn/pitch/roll angle
     * @param degrees the angle, in degrees
     */
    void setAngle(double degrees);
    
    /**
     * Set depth of recursion
     * @param depth depth of recursion
     */
    void setDepth(int depth);
    
    /**
     * Set diameter of segment
     * @param diameter the diameter
     */
    void setDiameter(double diameter);
    
    /**
     * Set diameter factor
     * @param diameterFactor the diameter factor
     */
    void setDiameterFactor(double diameterFactor);
    
    /**
     * Get the initial rule (the axiom)
     * @return the axiom
     */
    Rule getAxiom();
    
    /**
     * Get all rules
     * @return the rules
     */
    RuleSet getRules();
    
    /**
     * Get the turn/pitch/roll angle
     * @return the angle, in degrees
     */
    double getAngle();
    
    /**
     * Get depth of recursion
     * @return depth of recursion
     */
    int getDepth();
    
    /**
     * Get the generated model
     * @return generated model
     */
    Model *getModel();
    
protected:
    
    /**
     * Recursively apply the replacement rules
     * @param rule rule
     * @param level recursion level
     */
    void recursiveWalk(string rule, int level);
    
    // Parameters
    Rule _axiom;
    RuleSet _rules;
    int _depth;
    
    Model *_model;      // The active model
    Turtle *_turtle;    // The rendering turtle
};



#endif
